/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper;

/**
 * History:
 *
 * $Log: KRI.java,v $
 * Revision 1.3  2007/08/22 07:48:43  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2007/07/25 13:02:07  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1  2007/07/04 11:43:58  Trier
 * Neuordnung
 *
 * Revision 1.1.1.1  2007/06/20 09:56:26  Trier
 * dvs alles
 *
 * Revision 1.6  2006/04/28 05:48:49  Trier
 * Generics 1.5 verwendet
 *
 * Revision 1.5  2005/11/09 14:23:04  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.4  2005/09/09 07:37:28  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.3  2005/04/21 09:15:34  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2005/01/25 09:43:11  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1  2005/01/21 13:26:00  Trier
 * <Kein Kommentar eingegeben>
 *
 */

import java.util.ArrayList;
import java.util.List;
import java.util.WeakHashMap;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.DataModellMismatchException;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.DataModellObjectNotFoundException;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.LogosClientDavConnection;

/**
 * kapselt den DatenModellTyp kri.
 */
public class KRI extends ConfigurationObjectWrapper {

	private static final WeakHashMap<String, KRI> CACHE = new WeakHashMap<>();

	private static ConfigurationObjectType configurationObjectType;
	private static ConfigurationObjectType configurationObjectTypeOfSteuerModul;
	private static AttributeGroup atgGeraet;

	private String bezeichnung;
	private int knotenNummer;
	private String umsetzungsModul;
	private ArrayList<KriBus> busList;

	/**
	 * gibt die Bezeichnung des EAK zur&uuml;ck.
	 *
	 * @return String
	 */
	public String getBezeichnung() {
		return bezeichnung;
	}

	/**
	 * gibt die Knotennummer (OSI-7-Adresse) des Ger&auml;tes zur&uuml;ck.
	 *
	 * @return die Knotennummer;
	 */
	public int getKnotenNummer() {
		return knotenNummer;
	}

	/**
	 * gibt das für die Umsetzung des Protokolls (Lesen und Schreiben)
	 * verwendete Modul an diesem Knoten zur&uuml;ck.
	 *
	 * @return Klassennamen des UmsetzungsModuls.
	 */
	public String getUmsetzungsModul() {
		return umsetzungsModul;
	}

	/**
	 * erzeugt das WrapperObject f&uuml;r das Object mit der angegebenen Pid
	 *
	 * @see ConfigurationObjectWrapper#ConfigurationObjectWrapper(String pid)
	 *
	 * @param pid
	 *            String
	 * @throws DataModellMismatchException
	 *             ungültiges Datenmodell
	 * @throws DataModellObjectNotFoundException
	 *             ungültiges Datenmodell
	 */
	private KRI(final String pid) throws DataModellMismatchException, DataModellObjectNotFoundException {
		super(pid);
	}

	/**
	 * erzeugt das WrapperObject f&uuml;r das angegebene Object.
	 *
	 * @param confObj
	 *            ConfigurationObject
	 * @throws DataModellMismatchException
	 *             ungültiges Datenmodell
	 */
	private KRI(final ConfigurationObject confObj) throws DataModellMismatchException {
		super(confObj);
	}

	/**
	 * gibt das KRI mit der &uuml;bergebenen Pid zur&uuml;ck oder null, wenn es
	 * kein KRI mit der Pid gibt.
	 *
	 * @param pid
	 *            String
	 * @return SteuerModul
	 * @throws DataModellMismatchException
	 *             ungültiges Datenmodell
	 */
	public static synchronized KRI getInstance(final String pid) throws DataModellMismatchException {
		KRI kri = KRI.CACHE.get(pid);
		if (kri == null) {
			final Object object = LogosClientDavConnection.getInstance().getDataModel().getObject(pid);
			if ((object != null) && (object instanceof ConfigurationObject)) {
				kri = new KRI((ConfigurationObject) object);
			}
		}
		return kri;
	}

	@Override
	protected void initData(final ConfigurationObject confObj) throws DataModellMismatchException {

		// if (__AtgGeraet == null) {
		// DataModel model =
		// LogosClientDavConnection.getInstance().getDataModel();
		// __ConfigurationObjectTypeOfSteuerModul =
		// (ConfigurationObjectType)model.getType("typ.steuerModul");
		// __AtgGeraet = model.getAttributeGroup("atg.gerät");
		// }

		final Data oData = confObj.getConfigurationData(KRI.atgGeraet);
		if (oData != null) {
			bezeichnung = oData.getTextValue("Bezeichnung").getText();
			knotenNummer = oData.getUnscaledValue("KnotenNummer").intValue();
			umsetzungsModul = oData.getTextValue("UmsetzungsModul").getText();
		}
		final ObjectSet os = confObj.getObjectSet("AnschlussPunkteGerät");

		/*
		 * if (os != null) { List list =
		 * ConfigurationObjectWrapper.getKommunikationPartner(os.getElements());
		 * if ((list!=null) && (list.size()>0)) { _SMList = new
		 * ArrayList(list.size()); for (int i = 0; i < list.size(); i++) {
		 * ConfigurationObject smObj = (ConfigurationObject)list.get(i); if
		 * (smObj.isOfType(__ConfigurationObjectTypeOfSteuerModul)) {
		 * _SMList.add(new SteuerModul(smObj)); } } } }
		 */
		if (os != null) {
			final List<SystemObject> list = os.getElements();
			if ((list != null) && (list.size() > 0)) {
				final int iListSize = list.size();
				busList = new ArrayList<>(iListSize);
				for (int i = 0; i < iListSize; i++) {
					final ConfigurationObject busObj = (ConfigurationObject) list.get(i);
					final KriBus bus = new KriBus(busObj.getPid());
					busList.add(bus);
					final List<SystemObject> kpList = ConfigurationObjectWrapper.getKommunikationPartner(busObj);
					if ((kpList != null) && (kpList.size() > 0)) {
						for (int kpItemNr = 0; kpItemNr < kpList.size(); kpItemNr++) {
							final ConfigurationObject smObj = (ConfigurationObject) kpList.get(kpItemNr);
							if (smObj.isOfType(KRI.configurationObjectTypeOfSteuerModul)) {
								bus.addSteuerModul(new SteuerModul(smObj));
							}
						}
					}
				}
			}
		}
		KRI.CACHE.put(this.getPID(), this);
	}

	public KriBus[] getBuse() {
		if ((busList != null) && (busList.size() > 0)) {
			return busList.toArray(new KriBus[busList.size()]);
		}
		return null;
	}

	/**
	 * gibt ein Array aller Objekte zurück, die vom Typ typ.kri sind.
	 *
	 * @return Unterzentrale[]
	 * @throws DataModellMismatchException
	 *             ungültiges Datenmodell
	 */
	public static synchronized KRI[] getAll() throws DataModellMismatchException {

		final List<SystemObject> list = LogosClientDavConnection.getInstance().getDataModel().getType("typ.kri").getElements();
		final KRI[] retValue = new KRI[list.size()];
		for (int i = 0; i < list.size(); i++) {
			final ConfigurationObject confObj = (ConfigurationObject) list.get(i);
			final KRI kri = KRI.CACHE.get(confObj.getPid());
			if (kri == null) {
				retValue[i] = new KRI((ConfigurationObject) list.get(i));
			} else {
				retValue[i] = kri;
			}
		}
		return retValue;
	}

	@Override
	ConfigurationObjectType getType() {
		if (KRI.configurationObjectType == null) {
			final DataModel model = LogosClientDavConnection.getInstance().getDataModel();
			KRI.configurationObjectType = (ConfigurationObjectType) model.getType("typ.kri");

			KRI.configurationObjectTypeOfSteuerModul = (ConfigurationObjectType) model.getType("typ.steuerModul");
			KRI.atgGeraet = model.getAttributeGroup("atg.gerät");

			final List<SystemObject> list = KRI.configurationObjectType.getObjects();
			model.getConfigurationData(list, KRI.atgGeraet);

		}
		return KRI.configurationObjectType;
	}

	@Override
	public String getTreeString() {
		return getName();
	}

}
