/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper;

/**
 * History:
 *
 * $Log: MessQuerschnitt.java,v $
 * Revision 1.1  2007/07/04 11:44:06  Trier
 * Neuordnung
 *
 * Revision 1.1.1.1  2007/06/20 09:56:26  Trier
 * dvs alles
 *
 * Revision 1.3  2006/04/28 05:48:49  Trier
 * Generics 1.5 verwendet
 *
 * Revision 1.2  2005/01/21 13:26:16  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1.1.1  2004/03/23 13:14:19  Trier
 * Basis für LogosApps
 *
 * Revision 1.1.1.1  2004/03/23 07:59:52  Trier
 * vrz
 *
 */
import java.util.Arrays;
import java.util.List;
import java.util.WeakHashMap;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.DataModellMismatchException;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.DataModellObjectNotFoundException;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.LogosClientDavConnection;

/**
 * kapselt den DatenModellTyp messQuerschnitt
 */
public class MessQuerschnitt extends ConfigurationObjectWrapper {

	private static WeakHashMap<String, MessQuerschnitt> __Cache = new WeakHashMap<>();

	private static ConfigurationObjectType __ConfigurationObjectType;
	private static AttributeGroup __AtgMessQuerschnittAllgemein;

	private AuswerteQuerschnitt _AuswerteQuerschnitt;
	private String _ErsatzMessQuerschnittPID;
	private String _Typ;

	private Fahrstreifen[] _Fahrstreifen;

	/**
	 * gibt die zugehörigen Fahrstreifen sortiert von aussen nach innen (HFS,
	 * &Uuml;FS01, &Uuml;FS02 ..) zur&uuml;ck.
	 *
	 * @return Fahrstreifen[]
	 */
	public Fahrstreifen[] getFahrstreifen() {
		return _Fahrstreifen;
	}

	public SystemObject[] getFahrstreifenDMObjects() {
		final ObjectSet os = _ConfigurationObject.getObjectSet("FahrStreifen");
		return os.getElements().toArray((new SystemObject[os.getElements().size()]));
	}

	/**
	 * gibt den ErsatzMessQuerschnitt zurück, wenn dieser definiert wurde.
	 *
	 * @throws DataModellObjectNotFoundException
	 *             das angeforderte Objekt wurde nicht gefunden
	 * @throws DataModellMismatchException
	 *             das Datenmodell ist ungültig
	 * @return MessQuerschnitt
	 */
	public MessQuerschnitt getErsatzMessQuerschnitt()
			throws DataModellObjectNotFoundException, DataModellMismatchException {
		if (_ErsatzMessQuerschnittPID != null) {
			return MessQuerschnitt.getInstance(_ErsatzMessQuerschnittPID);
		}
		return null;
	}

	protected MessQuerschnitt(final String pid) throws DataModellMismatchException, DataModellObjectNotFoundException {
		super(pid);
	}

	protected MessQuerschnitt(final ConfigurationObject confObj) throws DataModellMismatchException {
		super(confObj);
	}

	public void setAuswerteQuerschnitt(final AuswerteQuerschnitt aq) {
		_AuswerteQuerschnitt = aq;
	}

	/**
	 * gibt die AuswerteQuerschnitt zur&uuml;ck, wenn dieses Objekt durch eine
	 * AuswerteQuerschnitt erzeugt wurde.
	 *
	 * @return AuswerteQuerschnitt
	 */
	public AuswerteQuerschnitt getAuswerteQuerschnitt() {
		return _AuswerteQuerschnitt;
	}

	@Override
	protected void initData(final ConfigurationObject confObj) throws DataModellMismatchException {

		final Data oData = confObj.getConfigurationData(MessQuerschnitt.__AtgMessQuerschnittAllgemein);
		if (oData != null) {
			_ErsatzMessQuerschnittPID = oData.getReferenceValue("ErsatzMessQuerschnitt").getSystemObject().getPid();

			if (oData.getUnscaledValue("Typ").isState()) {
				_Typ = oData.getUnscaledValue("Typ").getState().getName();
			}

		}

		final List<SystemObject> listFS = confObj.getObjectSet("FahrStreifen").getElements();
		_Fahrstreifen = new Fahrstreifen[listFS.size()];
		for (int fsNr = 0; (fsNr < listFS.size()); fsNr++) {
			try {
				_Fahrstreifen[fsNr] = Fahrstreifen.getInstance((ConfigurationObject) listFS.get(fsNr));
				_Fahrstreifen[fsNr].setMessQuerschnitt(this);
			} catch (final Exception e) {
				System.err.println(e);
				e.printStackTrace();
			}
		}
		Arrays.sort(_Fahrstreifen, new Fahrstreifen.FSComperatorVonAussenNachInnen());
		MessQuerschnitt.__Cache.put(this.getPID(), this);
	}

	/**
	 * gibt die Instance der MessQuerschnitt mit der PID zurück. Es wird eine
	 * DataModellObjectNotFoundException geworfen, wenn die PID nicht vorhanden ist
	 * oder das zur PID gehörende Objekt nich vom typ messQuerschnitt ist. Eine
	 * DataModellMismatchException wird geworfen, wenn Objektreferenzen auf falsche
	 * Typen verweisen bzw. diese nicht gesetzt sind.
	 *
	 * @param pid
	 *            String
	 * @throws DataModellObjectNotFoundException
	 *             das angeforderte Objekt wurde nicht gefunden
	 * @throws DataModellMismatchException
	 *             das Datenmodell ist ungültig
	 * @return MessQuerschnitt
	 */
	public static MessQuerschnitt getInstance(final String pid)
			throws DataModellObjectNotFoundException, DataModellMismatchException {

		MessQuerschnitt retValue = MessQuerschnitt.__Cache.get(pid);
		if (retValue == null) {
			retValue = new MessQuerschnitt(pid);
		}
		return retValue;
	}

	protected static MessQuerschnitt getInstance(final ConfigurationObject messQuerschnitt)
			throws DataModellMismatchException {

		MessQuerschnitt retValue = MessQuerschnitt.__Cache.get(messQuerschnitt.getPid());
		if (retValue == null) {
			retValue = new MessQuerschnitt(messQuerschnitt);
		}
		return retValue;
	}

	/**
	 * gibt die DatenModellObjekte vom typ messQuerschnitt zurück, wenn keiner
	 * dieser Objekte Fehler in den ObjektReferenzen aufweist.
	 * DataModellMismatchException wird geworfen, wenn Objektreferenzen auf falsche
	 * Typen verweisen bzw. diese nicht gesetzt sind.
	 *
	 * @throws DataModellMismatchException
	 *             das Datenmodell ist ungültig
	 * @return MessQuerschnitt[]
	 */
	public static MessQuerschnitt[] getAll() throws DataModellMismatchException {

		final List<SystemObject> list = LogosClientDavConnection.getInstance().getDataModel().getType("typ.messQuerschnitt")
				.getElements();
		final MessQuerschnitt[] retValue = new MessQuerschnitt[list.size()];
		for (int i = 0; i < list.size(); i++) {
			retValue[i] = new MessQuerschnitt((ConfigurationObject) list.get(i));
		}
		return retValue;
	}

	@Override
	public String toString() {
		final StringBuffer sb = new StringBuffer();
		sb.append("Messquerschnitt - PID: \"").append(this.getPID()).append("\"  ");
		sb.append("Name: \"").append(this.getName());
		return sb.toString();
	}

	public String getMultilineRepresentation(final int indent) {
		final String lf = System.getProperty("line.separator");
		final StringBuffer sIndent = new StringBuffer();
		for (int i = 0; i < indent; i++) {
			sIndent.append(' ');
		}
		final StringBuffer sb = new StringBuffer();
		sb.append(sIndent).append("PID: \"").append(this.getPID()).append("\"").append(lf);
		sb.append(sIndent).append("Name: \"").append(this.getName()).append("\"").append(lf);
		sb.append(sIndent).append("Typ: \"").append(_Typ).append("\"").append(lf);
		if (_ErsatzMessQuerschnittPID != null) {
			sb.append(sIndent).append("ErsatzMessQuerschnittPID: \"").append(_ErsatzMessQuerschnittPID).append("\"")
					.append(lf);
		} else {
			sb.append(sIndent).append("\"  _ErsatzMessQuerschnittPID ist null\"").append(lf);
		}
		sb.append(sIndent).append("  Fahrstreifen: ").append(lf);
		if ((_Fahrstreifen != null) && (_Fahrstreifen.length > 0)) {
			for (int i = 0; i < _Fahrstreifen.length; i++) {
				sb.append(sIndent).append("  Fahrstreifen " + (i + 1) + ": ").append(lf);
				sb.append(_Fahrstreifen[i].getMultilineRepresentation(4 + indent)).append(lf);
			}
		} else {
			sb.append("\"  keine Fahrstreifen vorhanden");
		}
		return sb.toString();

	}

	public String getMultilineRepresentation() {
		return getMultilineRepresentation(0);
	}

	@Override
	public String getTreeString() {
		return this.getName();
	}

	@Override
	ConfigurationObjectType getType() {
		if (MessQuerschnitt.__ConfigurationObjectType == null) {
			final DataModel model = LogosClientDavConnection.getInstance().getDataModel();
			MessQuerschnitt.__ConfigurationObjectType = (ConfigurationObjectType) model.getType("typ.messQuerschnitt");
			MessQuerschnitt.__AtgMessQuerschnittAllgemein = model.getAttributeGroup("atg.messQuerschnittAllgemein");
			final List<SystemObject> list = MessQuerschnitt.__ConfigurationObjectType.getElements();
			model.getConfigurationData(list, MessQuerschnitt.__AtgMessQuerschnittAllgemein);
		}
		return MessQuerschnitt.__ConfigurationObjectType;
	}

}
